#!/usr/bin/python

import os.path,sys
from numpy import *
from cmath import *

REMOVETMP = True

KL_EXEC = os.path.expanduser('~/build/kleinian/bin/kleinian')
if os.getenv('KLEINIAN'):
    KL_EXEC = os.path.expanduser(os.getenv('KLEINIAN'))
if not os.path.exists(KL_EXEC):
    KL_EXEC = os.path.expanduser('~/build/kleinian/kleinian')
if not os.path.exists(KL_EXEC):
    raise IOError('Could not find kleinian executable.')

import pexpect

GAP = '0.02'
MATRIXBOUND = '20'
RESOLUTION = '0.01'

#GAP = '0.004'
#MATRIXBOUND = '30'
#RESOLUTION = '0.002'

import re
#---------- Regular expression to identify a floating point----------#
simpfloatstr = r'-?\d+.\d+'
floatstr = r'[+-]?(((((\d+)?[.]\d+|\d+[.])|\d+)[eE][+-]?\d+)|((\d+)?[.]\d+|\d+[.]?))'
float_pat = re.compile(floatstr)

class Kleinian(object):
    def __init__(self,x,y,z=None,c=(-2.0+0.0j),gap=None,bound=None,res=None):
        self.x = complex(x)
        self.y = complex(y)
        self.z = z
        self.c = complex(c)
        if gap:
            self.gap = gap
        else:
            self.gap = GAP
        if bound:
            self.bound = bound
        else:
            self.bound = MATRIXBOUND
        if res:
            self.res = res
        else:
            self.res = RESOLUTION
        self.tempnam = os.tempnam(None,'kleinian')
        child = pexpect.spawn(KL_EXEC)
        #child.logfile = sys.stdout
        child.expect('SELECT:')
        child.sendline('lset')
        child.expect('Enter xmin,ymin,xmax,ymax:')
        child.sendline('-2,-2,2,2')
        child.expect('\):')
        child.sendline(self.tempnam + '.ps')
        child.expect('SELECT:')
        child.sendline('group')
        child.expect('SELECT:')
        child.sendline('twogen')
        child.expect(r'Default: tr b = ')
        child.sendline('(%.15f,%.15f)' % (self.x.real,self.x.imag))
        child.sendline('(%.15f,%.15f)' % (self.y.real,self.y.imag))
        child.sendline('(%.15f,%.15f)' % (self.c.real,self.c.imag))
        child.expect(r'The choices for trace\(ab\) are')
        child.expect(r'a:\s+\(\s*(?P<ar>'+floatstr+r')\s*,\s*(?P<ai>'+floatstr+r')\s*\)\s+or b:\s+\(\s*(?P<br>'+floatstr+r')\s*,\s*(?P<bi>'+floatstr+r')\s*\)')
        gd=child.match.groupdict()
        a = float(gd['ar']) + (1.0j)*float(gd['ai'])
        b = float(gd['br']) + (1.0j)*float(gd['bi'])
        if (self.z == None):
            child.sendline('a')
            self.z = a
        elif abs(self.z-a) < abs(self.z-b):
            child.sendline('a')
            self.z = a
        else:
            child.sendline('b')
            self.z = b
        child.expect('Enter tags')
        child.sendline('')

        # Get generators
        child.expect('SELECT:')
        child.sendline('gpcalc')
        child.expect('SELECT:')
        child.sendline('gens')
        child.expect('Generator word is abAB')
        # A
        child.expect(r'\s*a\s*=\s*\[\s*(?P<ar>'+floatstr+r')\+i\s+(?P<ai>'+simpfloatstr+r')\s+(?P<br>'+floatstr+r')\+i\s+(?P<bi>'+floatstr+r')\s*\]')
        gd=child.match.groupdict()
        a = float(gd['ar']) + (1.0j)*float(gd['ai'])
        b = float(gd['br']) + (1.0j)*float(gd['bi'])
        child.expect(r'\s*\[\s*(?P<cr>'+floatstr+r')\+i\s+(?P<ci>'+simpfloatstr+r')\s+(?P<dr>'+floatstr+r')\+i\s+(?P<di>'+floatstr+r')\s*\]')
        gd=child.match.groupdict()
        c = float(gd['cr']) + (1.0j)*float(gd['ci'])
        d = float(gd['dr']) + (1.0j)*float(gd['di'])
        self.A = matrix( [[a,b],[c,d]] )
        # B
        child.expect(r'\s*b\s*=\s*\[\s*(?P<ar>'+floatstr+r')\+i\s+(?P<ai>'+simpfloatstr+r')\s+(?P<br>'+floatstr+r')\+i\s+(?P<bi>'+floatstr+r')\s*\]')
        gd=child.match.groupdict()
        a = float(gd['ar']) + (1.0j)*float(gd['ai'])
        b = float(gd['br']) + (1.0j)*float(gd['bi'])
        child.expect(r'\s*\[\s*(?P<cr>'+floatstr+r')\+i\s+(?P<ci>'+simpfloatstr+r')\s+(?P<dr>'+floatstr+r')\+i\s+(?P<di>'+floatstr+r')\s*\]')
        gd=child.match.groupdict()
        c = float(gd['cr']) + (1.0j)*float(gd['ci'])
        d = float(gd['dr']) + (1.0j)*float(gd['di'])
        self.B = matrix( [[a,b],[c,d]] )

        self.gens = [self.A,self.B]
        child.sendline('up')
        # -----------------

        child.expect('SELECT:')
        child.sendline('file')
        child.sendline(self.tempnam + '.dat')
        child.expect('Enter gap between plotted points:')
        child.sendline(str(self.gap))
        child.expect('Enter matrix bound:')
        child.sendline(str(self.bound))
        child.expect('Enter plotting resolution:')
        child.sendline(str(self.res))
        child.expect('Enter starting word:')
        child.sendline('')
        child.expect('Enter  ending word:')
        child.sendline('')
        child.close()
        datafile = file(self.tempnam + '.dat')
        self.limset = [[float(t) for t in L.split()] for L in datafile.readlines()]
        self.limset = self.limset[:-1]
	sys.stderr.write('Limit set with %d vertices.\n' % len(self.limset))
        datafile.close()
        if REMOVETMP:
            os.unlink(self.tempnam + '.dat')
            os.unlink(self.tempnam + '.ps')
            os.unlink(self.tempnam + '.lbl')

if __name__ == '__main__':
    K = Kleinian(3.0,3.0,3.0)
    print len(K.limset)
    print K.A
    print K.B


