#!/usr/bin/python

import os.path,sys

REMOVETMP = True

Z_EXEC = os.path.expanduser('~/build/conformal/zipper/zipper')
I_EXEC = os.path.expanduser('~/build/conformal/zipper/inverse')
if not os.path.exists(Z_EXEC):
    raise IOError('Could not find \"zipper\" executable.')
if not os.path.exists(I_EXEC):
    raise IOError('Could not find \"inverse\" executable (part of \"zipper\").')

import pexpect

class Zipper(object):
    def __init__(self,verts,center,points):
        self.verts = verts
        self.center = center
        self.points = points

        # create and move to temporary directory
        self.oldcwd = os.getcwd()
        self.tempdir = os.tempnam(None,'zipper')
        os.mkdir(self.tempdir)
        os.chdir(self.tempdir)
        
        # write poly.dat
        polyfile = file(self.tempdir + '/poly.dat','wt')
        for v in self.verts:
            polyfile.write('%f %f\n' % (v[0],v[1]))
        polyfile.write('%f %f\n' % (self.center[0],self.center[1]))
        polyfile.close()

        # run zipper
        os.system(Z_EXEC)
        if not os.path.exists(self.tempdir + '/poly.pre'):
            raise IOError('zipper output not found (poly.pre).')
        if not os.path.exists(self.tempdir + '/poly.par'):
            raise IOError('zipper output not found (poly.par).')
        
        # write points.dat
        pointsfile = file(self.tempdir + '/points.dat','wt')
        for v in self.points:
            pointsfile.write('%f %f\n' % (v[0],v[1]))
        pointsfile.close()

        child = pexpect.spawn(I_EXEC)
        child.sendline('points.dat')
        child.sendline('points.pre')
        child.read()
        child.close()

        if not os.path.exists(self.tempdir + '/points.pre'):
            raise IOError('inverse output not found (points.pre).')

        prefile = file(self.tempdir + '/points.pre')
        self.images = [ [float(x) for x in L.split()] for L in prefile.readlines() ]
        prefile.close()

        if REMOVETMP:
            os.unlink(self.tempdir + '/poly.dat')
            os.unlink(self.tempdir + '/poly.pre')
            os.unlink(self.tempdir + '/poly.par')
            os.unlink(self.tempdir + '/points.dat')
            os.unlink(self.tempdir + '/points.pre')
            os.rmdir(self.tempdir)

if __name__ == '__main__':
    f = file('/home/ddumas/python/grlen/sample_poly.dat')
    v = [[float(x) for x in L.split()] for L in f.readlines() if len(L.split())==2]
    f.close()
    ctr = v[-1]
    v = v[:-1]
    Z = Zipper(v,ctr,[[-0.6,0.63],[-0.17,0.0]])
    print Z.images


